#!/usr/bin/python
##### Opto-electronic-oscillator phase-noise model - S53MV 05.04.2019 #####

import math,cmath
x=[]	#graph arrays
y=[]
z=[]

dfmin=10.0	#start plotting @ dfmin [Hz]
dfmax=1.0E+8	#stop plotting @ dfmax [Hz]
dfstep=1.0E-3	#frequency offset multiplication step (time versus accuracy)

Pc=1.0E-4	#minimum (photodiode) carrier power [W]
kb=1.38E-23	#Boltzmann constant [J/K]
Tr=290.0	#microwave resonator temperature [K]
Q=5.0E+3	#microwave resonator Q
fo=3.0E+9	#microwave resonator frequency [Hz]
m=15.0		#Q multipication factor in linear units
F=10.0		#amplifier noise figure in linear units
Toe=1.0E+5	#optoelectric delay-line temperature [K]
t=75.0E-6	#optoelectric time delay [seconds]

#ASSUMPTION: microwave resonator precisely tuned to the optoelectric time delay!
#NEGLECTED: (1) 1/f noise & (2) finite linewidth

def L(Pc,kb,Tr,Q,fo,m,F,Toe,t,df):	#theoretical Leeson OEO phase noise
	d=cmath.exp(-complex(0.0,1.0)*2*math.pi*df*t)
	a=1.0-d/(1.0+2.0*complex(0.0,1.0)*Q*m*df/fo)
	return 10.0*math.log10(kb*(Tr*m*m*F+Toe)/(2.0*Pc*abs(a*a)))

df=dfmin
while df<dfmax:		#frequency-offset loop
	x=x+[df]
	y=y+[L(Pc,kb,Tr,Q,fo,1.0,F,Toe,t,df)]	#no Q multiplier m=1
	z=z+[L(Pc,kb,Tr,Q,fo,m,F,Toe,t,df)]	#with Q multiplier m>1
	df=df*(1.0+dfstep)

import matplotlib.pyplot as plt		#plotting graphs
fig=plt.figure()
plt.semilogx(x,y,'c')
plt.semilogx(x,z,'r')
plt.title('OEO phase noise (Leeson analytical solution)')
plt.xlabel('Frequency offset [Hz]')
plt.ylabel('Phase-noise spectral density [dBc/Hz]')
plt.grid(True)
fig.savefig('oeo.png')
plt.show()

#program end

