#!/usr/bin/python3
##### Razširjeni Leeson za Qm-OEO - S53MV 14.02.2021 #####
#Privzeto: (1) brezhibno uglaševanje (2) ozka spektralna črta

dfmin=1e1   #začetni odmik [Hz]
dfmax=1e7   #končni odmik [Hz]
dfk=1.001   #frekvenčni množilni korak
P0=1e-4     #moč iz fotodiode [W]
kb=1.38e-23 #Boltzmann [J/K]
Ql=75000    #kvaliteta rezonatorja
tau=75e-6   #kasnilni vod [s]
f0=3e9      #osrednja frekvenca [Hz]
Td=5e5      #šumna temperatura [K]
fc=5e3      #prelom 1/f [Hz]

import math,cmath
x=[]        #polja za rezultat
y=[]
z=[]
df=dfmin
while df<dfmax:     #zanka po odmikih
    x=x+[df]
    d=cmath.exp(-1j*2*math.pi*df*tau)           #kasnilni vod
    a=1-d/(1+1j*2*Ql*df/f0)                     #prenos šuma
    L=kb*Td/(2*P0*abs(a**2))*(1+fc/df)          #razširjeni Leeson
    y+=[10*math.log10(L/(1+(2*Ql*df/f0)**2))]  #za sitom
    z+=[-10*math.log10(df)]
    df*=dfk

import matplotlib.pyplot as plt     #risanje grafa
fig=plt.figure()
plt.semilogx(x,z,'r--')
plt.semilogx(x,y,'b')
plt.axis([dfmin,dfmax,-180,-40])
plt.xlabel('Offset from carrier $\Delta f$ [Hz]')
plt.ylabel('Relative phase-noise spectral density $L(\Delta f)$ [dBc/Hz]')
plt.grid(c='k',linestyle='--')
fig.savefig('Qmoeo.png', dpi=600, transparent=True)
plt.show()

print('*** Konec ***')
